<?php
/**
* Analyses a modulegroup.
*/
class PhpdocModuleAnalyser extends PhpdocAnalyser {

	/**
	* Module data
	* @var	array
	*/
	var $modulegroup		= array();

	/**
	* List of all modules in the modulegroup
	* @var	array
	*/ 
	var $modulelist = array();

	/**
	* Sets the data of the modulegroup to analyse.
	* 
	* @param	array	Raw modulegroup data from the parser.
	* @access	public
	*/
	function setModulegroup($modulegroup) {
	
		$this->modulegroup = $modulegroup;	
		
	} // end func setModulegroup
	
	function analyse() {

		$this->flag_get = false;
		
		$this->buildModulelist();
		
		$this->updateAccessReturn();
		$this->updateBrothersSisters();
		
	} // end func analyse
	
	/**
	* Returns a module from the modulegroup or false if there are no more modules.
	*
	* @return	mixed		False if there no more modules in the modulegroup otherwise
	* 								an array with the data of a module.
	* @access	public
	*/
	function getModule() {
	
		if (!$this->flag_get) {
			reset($this->modulelist);
			$this->flag_get = true;
		}
			
		if (list($modulename, $group)=each($this->modulelist)) {
			
			$module = $this->modulegroup[$group][$modulename];
			unset($this->modulegroup[$group][$modulename]);
			return $module;
			
		} else {
		
			return false;
			
		}
		
	} // end func getModule
	
	/**
	* Builds an internal list of all modules in the modulegroup.
	* @see	$modulelist, $modulegroup
	*/
	function buildModulelist() {
	
		$this->modulelist = array();
		
		reset($this->modulegroup);
		while (list($group, $modules)=each($this->modulegroup)) {
		
			reset($modules);
			while (list($modulename, $data)=each($modules))
				$this->modulelist[$modulename] = $group;
				
		}
		
	}

		
	function updateBrothersSisters() {
	
		reset($this->modulelist);
		while (list($modulename, $group)=each($this->modulelist)) {
			$this->updateBrotherSisterElements($group, $modulename, "functions");
			$this->updateBrotherSisterElements($group, $modulename, "variables");
		}	
		
	} // end func updateBrothersSisters
	
	function updateBrotherSisterElements($group, $modulename, $type) {
		
		if (!isset($this->modulegroup[$group][$modulename][$type])) 
			return false;
			
		reset($this->modulegroup[$group][$modulename][$type]);
		while (list($elementname, $data)=each($this->modulegroup[$group][$modulename][$type])) {
			
			if (isset($data["brother"])) {

				$name = ( "functions" == $type ) ? substr($data["brother"], 0, -2) : substr($data["brother"], 1);
				$name = strtolower($name);

				if (!isset($this->modulegroup[$group][$modulename][$type][$name])) {
					$this->warn->addDocWarning($this->modulegroup[$group][$modulename]["filename"], $type, $elementname, "Brother '$name' is unknown. Tags gets ignored.", "mismatch");
					unset($this->modulegroup[$group][$modulename][$type][$elementname]["brother"]);
				} else {
					$this->modulegroup[$group][$modulename][$type][$elementname]["brother"] = $name;
				}

			}
			
		}
		
	} // end func updateBrotherSistersElements
	
	function updateAccessReturn() {
		
		reset($this->modulelist);
		while (list($modulename, $group)=each($this->modulelist)) {
		
			if (!isset($this->modulegroup[$group][$modulename]["access"]))
				$this->modulegroup[$group][$modulename]["access"] = "private";
				
			$this->updateAccessReturnElements($group, $modulename, "functions");
			$this->updateAccessReturnElements($group, $modulename, "variables");
			$this->updateAccessElements($group, $modulename, "consts");		
			
		}
				
	} // end func updateAccessReturn
	
	function updateAccessReturnElements($group, $modulename, $type) {
		
		if (!isset($this->modulegroup[$group][$modulename][$type]))
			return false;

		reset($this->modulegroup[$group][$modulename][$type]);
		while (list($elementname, $data)=each($this->modulegroup[$group][$modulename][$type])) {
		
			if (!isset($data["access"])) 
				$this->modulegroup[$group][$modulename][$type][$elementname]["access"] = "private";
				
			if (!isset($data["return"]))
				$this->modulegroup[$group][$modulename][$type][$elementname]["return"] = "void";
				
		}
				
	} // end func updateAccessReturnElements
	
	function updateAccessElements($group, $modulename, $type) {
		
		if (!isset($this->modulegroup[$group][$modulename][$type]))
			return false;
			
		reset($this->modulegroup[$group][$modulename][$type]);
		while (list($elementname, $data)=each($this->modulegroup[$group][$modulename][$type])) {
			
			if (!isset($data["access"])) 
				$this->modulegroup[$group][$modulename][$type][$elementname]["access"] = "private";
		
		}
		
	} // end func updateAccessElements
	
} // end class PhpdocModuleAnalyser
?>