<?php
/**
* Coordinates several Phpdoc Object to parse and render source files.
* 
* @access	public
*/
class Phpdoc extends PhpdocSetupHandler {

	/**
	* Result from the indexer
	* @var	array	$indexer
	* @see	render()
	*/
	var $indexer_result = array();
	
	/**
	* Print status messages
	*/
	var $flag_output = true;

	/**
	* Calls the command line handler if necessary.
	*
	* @global $argc, $PHP_SELF
	*/
	function Phpdoc() {
		global $argc, $PHP_SELF;
		
		$this->target = $PHP_SELF."apidoc/";
		
		if ($argc>1) 
			$this->handleArgv();
	} // end constructor

	
	/**
	* Starts the parser. 
	*
	* @return	bool		$ok
	* @throws	PhpdocError
	* @access	public
	* @see		
	*/
	function parse() {
		
		$this->warn = new PhpdocWarning;
		
		$errors = $this->checkStatus();
		if (0!=count($errors)) {
		
			reset($errors);
			while (list($k, $error)=each($errors))
				$this->err[] = new PhpdocError($error["msg"]."Errno = ".$error["errno"], 9, __FILE__, __LINE__);
				
			return false;
		}

		$this->outl("Parser starts...");
		
		// create some objects
		$fileHandler 		= new PhpdocFileHandler;
		$parser 				= new PhpdocParser(true);
		
		$classAnalyser	= new PhpdocClassAnalyser;
		$moduleAnalyser = new PhpdocModuleAnalyser;
		
		$classExporter	= new PhpdocXMLClassExporter;
		$classExporter->setPath($this->target);
		
		$moduleExporter = new PhpdocXMLModuleExporter;
		$moduleExporter->setPath($this->target);
		
		$indexer	= new PhpdocIndexer;		
		
		$indexExporter = new PhpdocXMLIndexExporter;
		$indexExporter->setPath($this->target);
				
		// This will change tomorrow!
		$parser->warn 				= $this->warn;
		$classAnalyser->warn 	= $this->warn;
		$moduleAnalyser->warn = $this->warn;
		$classExporter->warn 	= $this->warn;
		$moduleExporter->warn = $this->warn;
		$indexer->warn 				= $this->warn; 

		print $this->sourceDirectory."<hr>";
		
		$sourcefiles = $fileHandler->getFilesInDirectory($this->sourceDirectory, $this->sourceFileSuffix);
		$parser->setPhpSourcecodeFiles($fileHandler->get($sourcefiles));
			
		$this->outl("... preparse to find modulegroups and classtrees.");
		$parser->preparse();

		$this->outl("... parsing classes.");
		while ($classtree = $parser->getClassTree()) {

			$classAnalyser->setClasses( $classtree, $parser->current_baseclass );
			$classAnalyser->analyse();
			
			while ($class = $classAnalyser->getClass()) {
				$indexer->addClass($class);
				$classExporter->export($class);
			}
			
			if (floor(phpversion())>3) {
				$indexExporter->exportClasstree($indexer->getClasstree(), $parser->current_baseclass);
			} else {
				$classtree = $indexer->getClasstree();
				$base = $parser->current_baseclass;
				$indexExporter->exportClasstree($classtree, $base);
			}
			
		}
			
		$this->outl("... parsing modules.");
		while ($modulegroup = $parser->getModulegroup()) {	

			$moduleAnalyser->setModulegroup( $modulegroup );
			$moduleAnalyser->analyse();
				
			while ($module = $moduleAnalyser->getModule()) {
				$indexer->addModule($module);
				$moduleExporter->export($module);
			}

			if (floor(phpversion())>3) {
				$indexExporter->exportModulegroup($indexer->getModulegroup());
			} else {
				$modulegroup = $indexer->getModulegroup();
				$indexExporter->exportModulegroup($modulegroup);
			}
			
		}

		$this->outl("... writing packagelist.");
		
		if (floor(phpversion())>3) {
			$indexExporter->exportPackagelist($indexer->getPackages());
			$indexExporter->exportElementlist($indexer->getElementlist());
		} else {
			$packages = $indexer->getPackages();
			$indexExporter->exportPackagelist($packages);
			$elements = $indexer->getElementlist();
			$indexExporter->exportElementlist($elements);
		}

		$this->outl("Parser finished.");

		return true;
	} // end func parse
	
	/**
	* Renders the PHPDoc XML files as HTML files 
	*
	* @param	string 	Target directory for the html files
	* @param	string	Directory with the html templates
	* @return	bool		$ok
	* @throws	PhpdocError
	* @access	public
	*/
	function render($target="", $template="") {

		$this->outl("Starting to render...");
		
		if (""==$target)
			$target = $this->target;
		else 
			if (""!=$target && "/"!=substr($target, -1)) 
				$target.= "/";
				
		if (""==$template)
			$template = $this->templateRoot;
		else 
			if (""!=$template && "/"!=substr($template, -1)) 
				$template.= "/";
			
		$indexrenderer = new PhpdocHTMLIndexRenderer($target, $template, $this->application);
		$indexrenderer->generate();

		$classrenderer 	= new PhpdocHTMLClassRenderer($target, $template, $this->application);
		$modulerenderer = new PhpdocHTMLModuleRenderer($target, $template, $this->application);
		
		$fileHandler 		= new PhpdocFileHandler;
		$files = $fileHandler->getFilesInDirectory($target, "xml");

		$len = strlen($target);
		
		reset($files);
		while (list($k, $file)=each($files)) {
		
			if ("class_" == substr($file, $len, 6)) 
				$classrenderer->renderClass(substr($file, $len));

			if ("module_" == substr($file, $len, 7))
				$modulerenderer->renderModule(substr($file, $len));

			if ("classtree_" == substr($file, $len, 10)) 
				$indexrenderer->addClasstree(substr($file, $len));
				
			if ("modulegroup_" ==  substr($file, $len, 12)) 
				$indexrenderer->addModulegroup(substr($file, $len));
			
		}
		
		$indexrenderer->finishClasstree();
		$indexrenderer->finishModulegroup();
		
		$this->outl("Renderer finished.");
		
		return true;
	} // end func	render
	
} // end class Phpdoc
?>