<?php
/**
* Container for all kind of Warnings the parser/analyser recognizes
* 
* The base of the report generator module is this container. It's currently 
* pretty simple and will change later on...
*
* @author Ulf Wendel <ulf.wendel@redsys.de>
* @version 0.1alpha
*/
class PhpdocWarning extends PhpdocObject {
	
	/**
	* Hash of documentation failures.
	* @var	array	
	*/
	var $doc_warnings = array();
	
	/**
	* Hash of parser failures and warnings.
	* @var	array
	*/
	var $parser_warnings = array();
	
	/**
	* Counter containing the number of documentation warnings.
	* @var	int
	* @see	getNumDocWarnings(), getNumWarnings()
	*/
	var $num_doc_warnings = 0;
	
	/**
	* Conter containing the number of parser warnings.
	* @var	int
	* @see	getNumParserWarnings(), getNumWarnings()
	*/
	var $num_parser_warnings = 0;
	
	/**
	* Adds a warning to the list of class documentation failures.
	* @param	string	Name of the file that containts the error
	* @param	string	Kind of the element that caused the error: module, class, function, variable, use, const
	* @param	string	Name of the class/function/... that caused the warning
	* @param	string	Warning message itself
	* @para,	string	Type of the error: missing, mismatch, syntax, ...
	* @access	public
	* @see	addDocWarning()
	*/
	function addDocWarning($file, $elementtype, $elementname, $warning, $type="missing") {
	
		$this->doc_warnings[$file][$elementtype][$elementname][$type][] = $warning;
		$this->num_doc_warnings++;
		
	} // end func assClassDocWarning
	
	/**
	* Adds a parser warning to the list of class parsing failures.
	* @brother addDocWarning()
	* @see addParserWarning()
	*/
	function addParserWarning($file, $elementtype, $elementname, $warning, $type="missing") {
	
		$this->parser_warnings[$file][$elementtype][$elementname][$type][] = $warning;
		$this->num_parser_warnings++;
		
	} // end func addParserWarning
		
	/**
	* Adds a parser warning to the list of module parsing failures.
	* @brother addDocWarning()
	* @see addParserWarning()
	*/
	function addParserWarning($file, $elementtype, $elementname, $warning, $type="missing") {
		
		$this->parser_warnings[$file][$elementtype][$elementname][$type][] = $warning;
		$this->num_parser_warnings++;
		
	}	// end func addParserWarning
	
	/**
	* Returns the total number of documentation warnings.
	* @access	public
	* @see	getNumParserWarnings(), getNumWarnings()
	*/
	function getNumDocWarnings() {
		return $this->num_doc_warnings;
	} // end func getNumDocWarnings
	
	/**
	* Returns the total number of parser warnings.
	* @access	public
	* @see	getNumDocWarnings(), getNumWarnings()
	*/
	function getNumParserWarnings() {
		return $this->num_parser_warnings();
	} // end func getNumParserWarnings
	
	/**
	* Returns the total number of warnings (parser + documentation)
	* @access	public
	* @see	getNumDocWarnings(), getNumParserWarnings()
	*/
	function getNumWarnings() {
		return $this->num_doc_warnings + $this->num_parser_warnings;
	} // end func getNumWarnings
	
} // end class PhpdocWarning
?>